/* ---------------------------------------------------------------------
 * 
 * Giada - Your Hardcore Loopmachine 
 * 
 * ---------------------------------------------------------------------
 * 
 * Copyright (C) 2010-2012 Giovanni A. Zuliani | Monocasual
 * 
 * This file is part of Giada - Your Hardcore Loopmachine.
 * 
 * Giada - Your Hardcore Loopmachine is free software: you can 
 * redistribute it and/or modify it under the terms of the GNU General 
 * Public License as published by the Free Software Foundation, either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * Giada - Your Hardcore Loopmachine is distributed in the hope that it 
 * will be useful, but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Giada - Your Hardcore Loopmachine. If not, see 
 * <http://www.gnu.org/licenses/>.
 * 
 * ------------------------------------------------------------------ */




#include "ge_actionChannel.h"


extern Mixer    G_Mixer;
extern Conf	    G_Conf;





gActionChannel::gActionChannel(int x, int y, gdActionEditor *parent)
: Fl_Group(x, y, 200, 40), parent(parent), selected(NULL)
{
	

	for (unsigned i=0; i<recorder::frames.size; i++) {
		for (unsigned j=0; j<recorder::global.at(i).size; j++) {
			if (recorder::global.at(i).at(j)->chan == (unsigned)parent->chan) {

				

				if (recorder::frames.at(i) > G_Mixer.totalFrames)
					continue;

				

				if (G_Mixer.chanMode[parent->chan] == SINGLE_PRESS &&
				    recorder::global.at(i).at(j)->type == ACTION_KEYREL)
				{
					continue;
				}

				

				if (recorder::global.at(i).at(j)->type == ACTION_KILLCHAN &&
						G_Mixer.chanMode[parent->chan] == SINGLE_PRESS)
				{
					continue;
				}

				

				if (recorder::global.at(i).at(j)->type & (ACTION_MUTEON | ACTION_MUTEOFF))
					continue;

				int ax = x+((recorder::frames.at(i)/2)/parent->zoom);
				gAction *a = new gAction(
						ax,                                   
						y+4,                                  
						h()-8,                                
						recorder::frames.at(i),               
						i,                                    
						parent,                               
						false,                                
						recorder::global.at(i).at(j)->type);  
				add(a);
			}
		}
	}
	end(); 
}





gAction *gActionChannel::getSelectedAction() {
	for (int i=0; i<children(); i++) {
		int action_x  = ((gAction*)child(i))->x();
		int action_w  = ((gAction*)child(i))->w();
		if (Fl::event_x() >= action_x && Fl::event_x() <= action_x + action_w)
			return (gAction*)child(i);
	}
	return NULL;
}





void gActionChannel::updateActions() {

	

	gAction *a;
	for (int i=0; i<children(); i++) {

		a = (gAction*)child(i);
		int newX = x() + ((a->frame_a/2) / parent->zoom);

		if (G_Mixer.chanMode[parent->chan] == SINGLE_PRESS) {
			int newW = ((a->frame_b - a->frame_a) / 2 / parent->zoom);
			if (newW < gAction::MIN_WIDTH)
				newW = gAction::MIN_WIDTH;
			a->resize(newX, a->y(), newW, a->h());
		}
		else
			a->position(newX, a->y());
	}
}





void gActionChannel::draw() {

	

	w(parent->totalWidth);

	

	fl_rectf(x(), y(), parent->totalWidth, h(), COLOR_BG_MAIN);

	

	fl_color(COLOR_BD_0);
	fl_rect(x(), y(), parent->totalWidth, h());

	

	int    j = 0;
	bool end = false;

	

	for (int i=1; i<=parent->totalWidth && !end; i++) {
		int step = parent->zoom*i;
		while (j < step) {
			if (j > parent->totalFrames)     
				break;
			if (j % parent->framesPerBeat == 0) {                            
				fl_color(COLOR_BD_0);
				fl_line(i+x()-1, y()+1, i+x()-1, y()+h()-2);
			}
			if (j % parent->framesPerBar == 0 && i!=1 && i != parent->totalWidth) {    
				fl_color(COLOR_BG_2);
				fl_line(i+x()-1, y()+1, i+x()-1, y()+h()-2);
			}
			if (j == parent->framesPerBeats) {                      
				parent->coverX=i+x()-1;                               
				end = true;
			}
			j++;
		}
		j = step;
	}

	

	fl_rectf(parent->coverX, y()+1, parent->totalWidth-parent->coverX+x(), h()-2, COLOR_BG_1);
	draw_children();
}





int gActionChannel::handle(int e) {

	int ret = Fl_Group::handle(e);

	

	if (!active())
		return 1;

	switch (e) {

		case FL_DRAG: {
			if (selected != NULL) {   

				

				if (selected->onLeftEdge || selected->onRightEdge) {

					

					if (selected->onRightEdge) {

						int aw = Fl::event_x()-selected->x();
						int ah = selected->h();

						if (Fl::event_x() < selected->x()+gAction::MIN_WIDTH)
							aw = gAction::MIN_WIDTH;
						else
						if (Fl::event_x() > parent->coverX)
							aw = parent->coverX-selected->x();

						selected->size(aw, ah);
					}
					else {

						int ax = Fl::event_x();
						int ay = selected->y();
						int aw = selected->x()-Fl::event_x()+selected->w();
						int ah = selected->h();

						if (Fl::event_x() < x()) {
							ax = x();
							aw = selected->w()+selected->x()-x();
						}
						else
						if (Fl::event_x() > selected->x()+selected->w()-gAction::MIN_WIDTH) {
							ax = selected->x()+selected->w()-gAction::MIN_WIDTH;
							aw = gAction::MIN_WIDTH;
						}
						selected->resize(ax, ay, aw, ah);
					}
				}

				

				else {
					int real_x = Fl::event_x() - actionPickPoint;
					if (real_x < x())                                  
						selected->position(x(), selected->y());
					else
					if (real_x+selected->w() > parent->coverX)         
						selected->position(parent->coverX-selected->w(), selected->y());
					else
						selected->position(real_x, selected->y());
				}
				redraw();
			}
			ret = 1;
			break;
		}

		case FL_PUSH:	{

			if (Fl::event_button1()) {

				

				selected = getSelectedAction();

				if (selected == NULL) {
					gAction *a = new gAction(
							Fl::event_x(),                        
							y()+4,                                
							h()-8,                                
							((Fl::event_x()-x())*parent->zoom)*2, 
							recorder::frames.size-1,              
							parent,                               
							true,                                 
							ACTION_KEYPRESS);                     
					add(a);
					glue_setChannelWithActions(parent->chan); 
					redraw();
					ret = 1;
				}
				else {
					actionOriginalX = selected->x();
					actionOriginalW = selected->w();
					actionPickPoint = Fl::event_x() - selected->x();
					ret = 1;   
				}
			}
			else
			if (Fl::event_button3()) {
				gAction *a = getSelectedAction();
				if (a != NULL) {
					a->delAction();
					remove(a);
					delete a;
					glue_setChannelWithActions(parent->chan);
					redraw();
					ret = 1;
				}
			}
			break;
		}
		case FL_RELEASE: {

			if (selected == NULL) {
				ret = 1;
				break;
			}

			

			bool noChanges = false;
			if (actionOriginalX == selected->x())
				noChanges = true;
			if (G_Mixer.chanMode[parent->chan] == SINGLE_PRESS &&
					actionOriginalX+actionOriginalW != selected->x()+selected->w())
				noChanges = false;

			if (noChanges) {
				ret = 1;
				selected = NULL;
				break;
			}

			

			bool overlap = false;
			for (int i=0; i<children() && !overlap; i++) {

				

				if ((gAction*)child(i) == selected)
					continue;

				int action_x  = ((gAction*)child(i))->x();
				int action_w  = ((gAction*)child(i))->w();
				if (G_Mixer.chanMode[parent->chan] == SINGLE_PRESS) {

					

					 int start = action_x >= selected->x() ? action_x : selected->x();
					 int end   = action_x+action_w < selected->x()+selected->w() ? action_x+action_w : selected->x()+selected->w();
					 if (start < end) {
						selected->resize(actionOriginalX, selected->y(), actionOriginalW, selected->h());
						redraw();
						overlap = true;   
					}
				}
				else {
					if (selected->x() == action_x) {
						selected->position(actionOriginalX, selected->y());
						redraw();
						overlap = true;   
					}
				}
			}

			

			if (!overlap)
				selected->moveAction();
			selected = NULL;
			ret = 1;
			break;
		}
	}

	return ret;
}







const int gAction::MIN_WIDTH = 8;






gAction::gAction(int x, int y, int h, unsigned frame_a, unsigned index, gdActionEditor *parent, bool record, char type)
: Fl_Box     (x, y, MIN_WIDTH, h),
  selected   (false),
  index      (index),
  parent     (parent),
  type       (type),
  frame_a    (frame_a),
  onRightEdge(false),
  onLeftEdge (false)
{
	

	if (record)
		addAction();

	

	if (G_Mixer.chanMode[parent->chan] == SINGLE_PRESS) {
		frame_b = recorder::getEndActionFrame(parent->chan, ACTION_KEYREL, frame_a);
		if (frame_b == -1)
			frame_b = frame_a+4096;
		w((frame_b - frame_a)/parent->zoom/2);
	}
}





void gAction::draw() {

	


	if (G_Mixer.chanMode[parent->chan] == SINGLE_PRESS)
		if (w() < MIN_WIDTH)
			w(MIN_WIDTH);


	int color;
	if (selected)  
		color = COLOR_BD_1;
	else
		color = COLOR_BG_2;

	if (type == ACTION_KILLCHAN)
		fl_rect(x(), y(), w(), h(), (Fl_Color) color);
	else {
		fl_rectf(x(), y(), w(), h(), (Fl_Color) color);
		if (G_Mixer.chanMode[parent->chan] != SINGLE_PRESS) { 
			if (type == ACTION_KEYPRESS)
				fl_rectf(x()+3, y()+h()-11, 2, 8, COLOR_BD_0);
			else
			if  (type == ACTION_KEYREL)
				fl_rectf(x()+3, y()+3, 2, 8, COLOR_BD_0);
		}
	}

}





int gAction::handle(int e) {

	

	int ret = 0;

	switch (e) {

		case FL_ENTER: {
			selected = true;
			ret = 1;
			redraw();
			break;
		}
		case FL_LEAVE: {
			fl_cursor(FL_CURSOR_DEFAULT, FL_WHITE, FL_BLACK);
			selected = false;
			ret = 1;
			redraw();
			break;
		}
		case FL_MOVE: {

			

			if (G_Mixer.chanMode[parent->chan] == SINGLE_PRESS) {
				onLeftEdge  = false;
				onRightEdge = false;
				if (Fl::event_x() >= x() && Fl::event_x() < x()+4) {
					onLeftEdge = true;
					fl_cursor(FL_CURSOR_WE, FL_WHITE, FL_BLACK);
				}
				else
				if (Fl::event_x() >= x()+w()-4 && Fl::event_x() <= x()+w()) {
					onRightEdge = true;
					fl_cursor(FL_CURSOR_WE, FL_WHITE, FL_BLACK);
				}
				else
					fl_cursor(FL_CURSOR_DEFAULT, FL_WHITE, FL_BLACK);
			}
		}
	}

	return ret;
}





void gAction::addAction() {

	

	if (G_Mixer.chanMode[parent->chan] == SINGLE_PRESS) {
		recorder::rec(parent->chan, ACTION_KEYPRESS, frame_a);
		recorder::rec(parent->chan, ACTION_KEYREL, frame_a+4096);
		
	}
	else {
		recorder::rec(parent->chan, ACTION_KEYPRESS, frame_a);
		
	}

	recorder::sortActions();

	index++; 
}





void gAction::delAction() {

	

	if (G_Mixer.chanMode[parent->chan] == SINGLE_PRESS) {
		recorder::deleteAction(parent->chan, frame_a, ACTION_KEYPRESS);
		recorder::deleteAction(parent->chan, frame_b, ACTION_KEYREL);
	}
	else
		recorder::deleteAction(parent->chan, frame_a, type);

	

	fl_cursor(FL_CURSOR_DEFAULT, FL_WHITE, FL_BLACK);
}





void gAction::moveAction() {

	

	delAction();

	

	frame_a = (x()-parent->ac->x())*parent->zoom*2;
	recorder::rec(parent->chan, type, frame_a);

	if (G_Mixer.chanMode[parent->chan] == SINGLE_PRESS) {
		frame_b = (x()+w()-parent->ac->x())*parent->zoom*2;;
		recorder::rec(parent->chan, ACTION_KEYREL, frame_b);
	}

	recorder::sortActions();
}
