/* ---------------------------------------------------------------------
 * 
 * Giada - Your Hardcore Loopmachine 
 * 
 * ---------------------------------------------------------------------
 * 
 * Copyright (C) 2010-2012 Giovanni A. Zuliani | Monocasual
 * 
 * This file is part of Giada - Your Hardcore Loopmachine.
 * 
 * Giada - Your Hardcore Loopmachine is free software: you can 
 * redistribute it and/or modify it under the terms of the GNU General 
 * Public License as published by the Free Software Foundation, either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * Giada - Your Hardcore Loopmachine is distributed in the hope that it 
 * will be useful, but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Giada - Your Hardcore Loopmachine. If not, see 
 * <http://www.gnu.org/licenses/>.
 * 
 * ------------------------------------------------------------------ */



#ifndef MIXER_H
#define MIXER_H

#include <stdlib.h>
#include <pthread.h>
#include "init.h"
#include "const.h"
#include "kernelAudio.h"
#include "wave.h"
#include "gui_utils.h"
#include "recorder.h"
#include "pluginHost.h"


class Mixer {
public:

	Mixer();
	~Mixer();
	void init();
	int  close();

	void loadWave(class Wave *w, int chan);
	void freeWave(int chan);

	void chanStop(int chan);
	void chanReset(int chan);

	void fadein(int chan);

	
	void fadeout(int chan, int actionPostFadeout=DO_STOP);

	void xfade(int chan);

	
	int getChanPos(int chan);

	
	static int masterPlay(
		void *out_buf, void *in_buf, unsigned n_frames,
		double streamTime, RtAudioStreamStatus status, void *userData
	);
	int __masterPlay(void *out_buf, void *in_buf, unsigned n_frames);

	
	void updateFrameBars();

	
	bool isSilent();

	void rewind();

	
	void updateQuanto();

	
	bool hasLogicalSamples();

	
	bool hasEditedSamples();

	
	void setPitch(int chan, float val);

	void setChanStart(int chan, unsigned val);
	void setChanEnd  (int chan, unsigned val);


	


	bool     running;
	Wave    *chan          [MAX_NUM_CHAN];
	float   *vChan				 [MAX_NUM_CHAN];	
	int      chanStatus    [MAX_NUM_CHAN];	
	unsigned chanTracker   [MAX_NUM_CHAN];  
	unsigned chanStart     [MAX_NUM_CHAN];
	unsigned chanEnd       [MAX_NUM_CHAN];
	unsigned chanStartTrue [MAX_NUM_CHAN];	
	unsigned chanEndTrue   [MAX_NUM_CHAN];	
	float    chanVolume    [MAX_NUM_CHAN];
  float    chanPitch     [MAX_NUM_CHAN];
	float    chanBoost     [MAX_NUM_CHAN];
	float 	 chanPanLeft   [MAX_NUM_CHAN];
	float 	 chanPanRight  [MAX_NUM_CHAN];
	unsigned chanMode      [MAX_NUM_CHAN];  

	
	bool     chanMute      [MAX_NUM_CHAN];

	float    chanMuteVol   [MAX_NUM_CHAN];
	bool     chanQWait     [MAX_NUM_CHAN];  
	int 		 chanRecStatus [MAX_NUM_CHAN];  
	float    chanFadein    [MAX_NUM_CHAN];
	bool     fadeoutOn     [MAX_NUM_CHAN];	
	float    fadeoutVol    [MAX_NUM_CHAN];  
	unsigned fadeoutTracker[MAX_NUM_CHAN];  
	float    fadeoutStep   [MAX_NUM_CHAN];  
  int      fadeoutType   [MAX_NUM_CHAN];  
  int			 fadeoutEnd    [MAX_NUM_CHAN];  

	unsigned frameSize;
	float    outVol;
	float    inVol;
	float    peakOut;
	float    peakIn;
	unsigned quanto;
	char     quantize;
	bool		 metronome;
	float    bpm;
	int   	 bars;
	int      beats;
	int      waitRec; 

	bool docross;			
	bool rewindWait;	

	int framesPerBar;
	int framesPerBeat;
	int totalFrames;
	int actualFrame;

#define TICKSIZE 38
	static float tock[TICKSIZE];
	static float tick[TICKSIZE];
	int  tickTracker, tockTracker;
	bool tickPlay, tockPlay; 

	
	int chanInput;

	
	int inputTracker;

	pthread_mutex_t mutex_recs;
	pthread_mutex_t mutex_plugins;


private:

	
	void calcFadeoutStep(int ch);
};

#endif
