/* ---------------------------------------------------------------------
 * 
 * Giada - Your Hardcore Loopmachine 
 * 
 * ---------------------------------------------------------------------
 * 
 * Copyright (C) 2010-2012 Giovanni A. Zuliani | Monocasual
 * 
 * This file is part of Giada - Your Hardcore Loopmachine.
 * 
 * Giada - Your Hardcore Loopmachine is free software: you can 
 * redistribute it and/or modify it under the terms of the GNU General 
 * Public License as published by the Free Software Foundation, either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * Giada - Your Hardcore Loopmachine is distributed in the hope that it 
 * will be useful, but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Giada - Your Hardcore Loopmachine. If not, see 
 * <http://www.gnu.org/licenses/>.
 * 
 * ------------------------------------------------------------------ */



#include "recorder.h"

extern Mixer G_Mixer;
extern Patch f_patch;
extern Conf	 G_Conf;

namespace recorder {

gVector<int> frames;
gVector< gVector<_action*> > global;
gVector<_action*>  actions;

bool active = false;
bool chanActive[MAX_NUM_CHAN];
bool chanEvents[MAX_NUM_CHAN];
bool sortedActions = false;




void init() {
	sortedActions = false;
	active = false;
	clearAll();
}





bool canRec(int ch) {

	

	if (!active || !G_Mixer.running || G_Mixer.chanInput == ch || G_Mixer.chan[ch] == NULL)
		return 0;
	return 1;
}





void rec(unsigned c, char act, int frame) {

	

	_action *a = (_action*) malloc(sizeof(_action));
	a->chan  = c;
	a->type  = act;

	

	int frameToExpand = frames.size;
	for (int i=0; i<frameToExpand; i++)
		if (frames.at(i) == frame) {
			frameToExpand = i;
			break;
		}

	

	if (frameToExpand == (int) frames.size) {
		frames.add(frame);
		global.add(actions);							
		global.at(global.size-1).add(a);	
	}
	else {

		

		for (unsigned t=0; t<global.at(frameToExpand).size; t++)
			if (global.at(frameToExpand).at(t)->chan == c && global.at(frameToExpand).at(t)->type == act)
				return;
		global.at(frameToExpand).add(a);		
	}

	

	chanEvents[c] = true;
	sortedActions = false;

	printf("[REC] action %d recorded on frame %d, chan %d\n", act, frame, c);
	
}





void clearChan(unsigned ch) {

	printf("[REC] clearing chan %d...\n", ch);

	for (unsigned i=0; i<global.size; i++) {	
		unsigned j=0;
		while (true) {
			if (j == global.at(i).size) break; 	  
			if (global.at(i).at(j)->chan == ch)	{
				free(global.at(i).at(j));
				global.at(i).del(j);
			}
			else
				j++;
		}
	}
	chanEvents[ch] = false;
	optimize();
	
}





void clearAction(unsigned ch, char act) {
	printf("[REC] clearing action %d from chan %d...\n", act, ch);
	for (unsigned i=0; i<global.size; i++) {						
		unsigned j=0;
		while (true) {                                   
			if (j == global.at(i).size)
				break;
			if (global.at(i).at(j)->chan == ch && (act & global.at(i).at(j)->type) == global.at(i).at(j)->type)	{ 
				free(global.at(i).at(j));
				global.at(i).del(j);
			}
			else
				j++;
		}
	}
	chanEvents[ch] = false; 
	optimize();
	chanHasEvents(ch);
	
}





void deleteAction(unsigned chan, int frame, char type) {

	

	bool found = false;
	for (unsigned i=0; i<frames.size && !found; i++) {
		if (frames.at(i) == frame) {

			

			for (unsigned j=0; j<global.at(i).size; j++)
				if (global.at(i).at(j)->chan == chan &&
				    global.at(i).at(j)->type == type)
				 {
					free(global.at(i).at(j));
					global.at(i).del(j);
					found = true;
					break;
				}
		}
	}
	if (found) {
		optimize();
		chanHasEvents(chan);
		printf("[REC] deleted action type=%d at frame %d from chan %d\n", type, frame, chan);
	}
}





void clearAll() {
	while (global.size > 0) {
		for (unsigned i=0; i<global.size; i++) {
			for (unsigned k=0; k<global.at(i).size; k++)
				free(global.at(i).at(k));									
			global.at(i).clear();												
			global.del(i);
		}
	}

	for (unsigned i=0; i<MAX_NUM_CHAN; i++) {
		chanEvents[i] = false;
		chanActive[i] = false;
	}

	global.clear();
	frames.clear();
}





void optimize() {

	

	unsigned i = 0;
	while (true) {
		if (i == global.size) return;
		if (global.at(i).size == 0) {
			global.del(i);
			frames.del(i);
		}
		else
			i++;
	}

	sortActions();
}





void sortActions() {
	if (sortedActions)
		return;
	for (unsigned i=0; i<frames.size; i++)
		for (unsigned j=0; j<frames.size; j++)
			if (frames.at(j) > frames.at(i)) {
				frames.swap(j, i);
				global.swap(j, i);
			}
	sortedActions = true;
	
}





void updateBpm(float oldval, float newval, int oldquanto) {

	for (unsigned i=0; i<frames.size; i++) {

		float frame  = ((float) frames.at(i)/newval) * oldval;
		frames.at(i) = (int) frame;

		
		

		if (frames.at(i) != 0) {
			int scarto = oldquanto % frames.at(i);
			if (scarto > 0 && scarto <= 6)
				frames.at(i) = frames.at(i) + scarto;
		}

		

		if (frames.at(i) % 2 != 0)
			frames.at(i)++;
	}
	
}





void updateSamplerate(int systemRate, int patchRate) {

	

	if (systemRate == patchRate)
		return;

	printf("[REC] systemRate (%d) != patchRate (%d), converting...\n", systemRate, patchRate);

	float ratio = systemRate / (float) patchRate;
	for (unsigned i=0; i<frames.size; i++) {

		float newFrame = frames.at(i);
		newFrame = floorf(newFrame * ratio);

		printf("[REC]    oldFrame = %d, newFrame = %d\n", frames.at(i), (unsigned) newFrame);

		frames.at(i) = (unsigned) newFrame;

		if (frames.at(i) % 2 != 0)
			frames.at(i)++;
	}
}





void expand(int old_fpb, int new_fpb) {

	

	unsigned pass = (int) (new_fpb / old_fpb) - 1;
	if (pass == 0) pass = 1;

	unsigned init_fs = frames.size;

	for (unsigned z=1; z<=pass; z++) {
		for (unsigned i=0; i<init_fs; i++) {
			unsigned newframe = frames.at(i) + (old_fpb*z);
			frames.add(newframe);
			global.add(actions);
			for (unsigned k=0; k<global.at(i).size; k++)
				rec(global.at(i).at(k)->chan, global.at(i).at(k)->type, newframe);
		}
	}
	printf("[REC] expanded recs\n");
	
}





void shrink(int new_fpb) {

	

	unsigned i=0;
	while (true) {
		if (i == frames.size) break;

		if (frames.at(i) >= new_fpb) {
			for (unsigned k=0; k<global.at(i).size; k++)
				free(global.at(i).at(k));			
			global.at(i).clear();								
			global.del(i);											
			frames.del(i);											
		}
		else
			i++;
	}
	optimize();
	printf("[REC] shrinked recs\n");
	
}





void enableRead(int c) {
	chanActive[c] = true;
}





void disableRead(int c) {
	chanActive[c] = false;
	if (G_Conf.recsStopOnChanHalt)
		mh_killChan(c);
}





void chanHasEvents(unsigned ch) {
	if (global.size == 0) {
		chanEvents[ch] = false;
		return;
	}
	for (unsigned i=0; i<global.size && !chanEvents[ch]; i++) {
		for (unsigned j=0; j<global.at(i).size && !chanEvents[ch]; j++) {
			if (global.at(i).at(j)->chan == ch)
				chanEvents[ch] = true;
		}
	}
}





int getStartActionFrame(unsigned chan, char action, int frame) {

	

	for (unsigned i=frames.size; i>0; i--)
		for (unsigned j=0; j<global.at(i-1).size; j++)
			if (global.at(i-1).at(j)->chan == chan &&
					global.at(i-1).at(j)->type == action)
			{
				if (frames.at(i-1) < frame) {
					printf("[REC] start Action frame found = %d\n", frames.at(i-1));
					return frames.at(i-1);
				}
				else
				if (frames.at(i-1) == frame) {
					printf("[REC start Action & end Action collision at %d!\n", frame);
					return -2;
				}
			}

	puts("[REC] start Action frame NOT found, suspected ring loop!");
	return -1;
}





int getEndActionFrame(unsigned chan, char action, int frame) {

	for (unsigned i=0; i<frames.size; i++)
		for (unsigned j=0; j<global.at(i).size; j++)
			if (global.at(i).at(j)->chan == chan &&
					global.at(i).at(j)->type == action &&
					frames.at(i) > frame)
			{
				printf("[REC] end Action frame found | frame=%d, index=%d\n", frames.at(i), i);
				return frames.at(i);
			}

	puts("[REC] end Action frame NOT found, suspected ring loop!");
	return -1;
}





void print() {
	printf("[REC] ** print debug **\n");
	for (unsigned i=0; i<global.size; i++) {
		printf("      [f.%d]", frames.at(i));
		for (unsigned j=0; j<global.at(i).size; j++) {
			printf("[a.%d|c.%d]", global.at(i).at(j)->type, global.at(i).at(j)->chan);
		}
		printf("\n");
	}
}

} 
